# Шпаргалка по командам samba-tool

- [Шпаргалка по командам samba-tool](#шпаргалка-по-командам-samba-tool)
  - [Пользователи и группы](#пользователи-и-группы)
    - [Создать пользователя](#создать-пользователя)
    - [Отключить устаревание пароля для пользователя](#отключить-устаревание-пароля-для-пользователя)
    - [Удалить пользователя](#удалить-пользователя)
    - [Создать группу](#создать-группу)
    - [Удалить группу](#удалить-группу)
    - [Добавить пользователей в группу](#добавить-пользователей-в-группу)
    - [Удалить пользователей из группы](#удалить-пользователей-из-группы)
    - [Показать пользователей группы](#показать-пользователей-группы)
  - [Работа со встроенным в Samba 4 DNS сервером](#работа-со-встроенным-в-samba-4-dns-сервером)
    - [Запрос списка зон](#запрос-списка-зон)
    - [Запрос информации о DNS зоне](#запрос-информации-о-dns-зоне)
    - [Запрос всех DNS записей](#запрос-всех-dns-записей)
    - [Смена типа DNS сервера (Встроенный/Bind9)](#смена-типа-dns-сервера-встроенныйbind9)
    - [Создание записи типа А](#создание-записи-типа-а)
    - [Создание зоны обратного просмотра](#создание-зоны-обратного-просмотра)
    - [Создание записи в зоне обратного просмотра](#создание-записи-в-зоне-обратного-просмотра)
    - [Запрос записей в зоне обратного просмотра](#запрос-записей-в-зоне-обратного-просмотра)
    - [Специфичные запросы](#специфичные-запросы)

***

Больше информации здесь - <https://xinux.net/index.php/Samba-tool>

***

## Пользователи и группы

### Создать пользователя

```bash
amba-tool user create username
```

```bash
samba-tool user create ivanov P@ssw0rd \
    --login-shell=/bin/bash \
    --gecos='Иван Иванов' \
    --given-name=Иван --surname=Иванов 
```

### Отключить устаревание пароля для пользователя

```bash
samba-tool user setexpiry ivanov --noexpiry 
```

### Удалить пользователя

```bash
samba-tool user delete username
```

### Создать группу

```bash
samba-tool group add groupname
```

### Удалить группу

```bash
samba-tool group delete groupname
```

### Добавить пользователей в группу

```bash
samba-tool group addmembers "Domain Users" user[,otheruser[,thirduser[,...]]]
```

### Удалить пользователей из группы

```bash
samba-tool group removemembers "Domain Users" user[,otheruser[,thirduser[,...]]]
```

### Показать пользователей группы

```bash
samba-tool group listmembers "Domain Users"
```

## Работа со встроенным в Samba 4 DNS сервером

### Запрос списка зон

```bash
samba-tool dns zonelist 127.0.0.1 -U Administrator
```

### Запрос информации о DNS зоне

```bash
samba-tool dns zoneinfo 127.0.0.1 local.acisi.ru -U Administrator
```

### Запрос всех DNS записей

```bash
samba-tool dns query 127.0.0.1 local.acisi.ru @ ALL -U Administrator
```

### Смена типа DNS сервера (Встроенный/Bind9)

```bash
samba_upgradedns --dns-backend=[BIND9_DLZ|SAMBA_INTERNAL]
```

### Создание записи типа А

```bash
samba-tool dns add 10.10.120.24 alt.lan COMP2 A 10.10.120.25 -Uadministrator
```

### Создание зоны обратного просмотра

для сети 192.168.13.0/24

```bash
samba-tool dns zonecreate 127.0.0.1 13.168.192.in-addr.arpa -Uadministrator
```

### Создание записи в зоне обратного просмотра

для узла dc1.alt.lan с адресом 192.168.13.139

```bash
samba-tool dns add 127.0.0.1 13.168.192.in-addr.arpa 139 PTR dc1.alt.lan -Uadministrator --password=P@ssw0rd
```

### Запрос записей в зоне обратного просмотра

```bash
samba-tool dns query 127.0.0.1 13.168.192.in-addr.arpa @ ALL -U Administrator
```

### Специфичные запросы

```bash
dig _msdcs.local.acisi.ru @127.0.0.1 SOA
```
